function updateTimestamps() {
  $('[rel=timestamp]').each(function() {
    $(this).html(
      DateHelper.time_ago_in_words_with_parsing(
        $(this).attr('title')
      )
    );
  });
}

setInterval(updateTimestamps, 60000);
setInterval(updateCounts, 30000);

var Application = chrome.extension.getBackgroundPage().getApplication();

var current_timeline = 'home';

function updateCounts() {
  $.each(Application.counts, function(key, value) {
    var menuItem = $('a[href=#' + key + ']');
    menuItem.find('span.count').remove();
    if (value > 0) menuItem.append(' <span class="count">' + value + '</span>')
  });
}

function currentTimeline() { return current_timeline; }

function renderUpdates(updates, is_new) {
  var group = $('<div class="update_group"></div>');
	
	$.each(updates, function() {
    var json = Presenters.update(this, current_timeline);
    var html = Mustache.to_html(Views.update, json, {paste:Views.paste, attachments:Views.attachments});
    group.append(html);
  });
  
  $('#stream').prepend(group);
  
  anyNew();
  
  $('div.update .body a').attr('target','_presently');
  
  $('div.spinner').fadeOut();
  
  updateTimestamps();
  updateCounts();
}

function anyNew() {
	if ($('.update.new').length == 0) {
    $('a[rel=mark_as_read]').addClass('disabled');
  } else {
    $('a[rel=mark_as_read]').removeClass('disabled');
  }
}

function updateDisplay() {
  $("style.display").remove();      
  if (localStorage['display_preference'] == 'new')
    $("head").append('<style class="display" type="text/css">div.update{display:none;} div.update.new{display:block;}</style>');
}

$(function() {
  updateDisplay();
  
  if (!localStorage['username'] || !localStorage['access_token'] || !localStorage['access_secret']) {
    $('body').html('<div class="setup"><h2>Welcome to Skeet!</h2><p align="center"><img src="app_icon_128.png"/></p><h3>You haven\'t set up your account yet.</h3> <a onclick="javascript:window.close();" href="' + chrome.extension.getURL('options.html') + '" target="_blank">Go to the options page</a></div>');
  }
  
  renderUpdates(Application.updates.home, false);
  
  $('a[rel=mark_as_read]').click(function() {
    $(this).addClass('disabled');
    $('div.update.new').removeClass('new');
    localStorage['last_read_' + current_timeline + '_id'] = $('div.update:first').attr('data-id');
    Application.markAsRead(current_timeline);
    updateDisplay();
    updateCounts();
    return false;
  });
  
  $('#all_or_new a').click(function() {
    $('#all_or_new a').removeClass('active');
    $(this).addClass('active');
    localStorage['display_preference'] = $(this).attr('href').replace('#','');
    updateDisplay();
    return false;
  });
  
  $('#update_form').submit(function() {
    $('#update_form button')
      .html('Sending')
      .addClass('sending');
      
    $('#update_form textarea, #update_form button')
      .attr('disabled','disabled');   
             
    Presently.update($('#update_form textarea').val(), function() {
      $('#update_form button')
        .html('Update')
        .removeClass('sending');
      $('#update_form button, #update_form textarea')
        .removeAttr('disabled');
      $('#update_form textarea').val('');
    });
    
    $('a[rel=refresh]').click();
    
    return false;
  });

	$('#navigation a').live('click', function() {
		if (current_timeline == $(this).attr('href').substr(1)) return false;
		if (current_timeline == 'search') $('#stream').data('search_cache', $('#stream').html());
		$(this).parents('#navigation').find('li').removeClass('current');
		$(this).parent().addClass('current');
		current_timeline = $(this).attr('href').substr(1);
		$(document).trigger('navigation.change', current_timeline);
		anyNew();
		renderUpdates(Application.updates[current_timeline], false);
	});
	
	$(document).bind('navigation.change', function(e, timeline) {
	  if (timeline == 'search') {
	    $('#search_field').val('');
	    $('#search_form').show();
	    $('#all_or_new, a[rel=mark_as_read]').hide();
	    $('#search_field').focus();
	    if (Application.last_search == '') $('a[rel=refresh]').addClass('disabled');
	    Application.last_search ? $('#stream').html(Application.last_search) : $('#stream').html('');
	  } else {
	    $('#stream').html('');
	    $('#all_or_new, a[rel=mark_as_read]').show();
	    $('#search_form').hide();
	    $('a[rel=refresh]').removeClass('disabled');
	  }
	});
  
  function updateCounter(area) {
    var remaining = 140 - $(this).val().length;
    if (remaining < 140)
      $('#update_form button')
        .data('count', remaining)
        .html(remaining);
    else
      $('#update_form button').html('Update');
  }
  
  $('#search_form').live('submit', function() {
    $('#stream').html('');
    $('.spinner').fadeIn('fast');
    var q = $('#search_field').val()
    Presently.search(q, function(search_results) {
      $('.spinner').fadeOut('fast');
      var search_updates = []
      $.each(search_results.results, function() {
        search_updates.push(Presenters.search_result(this));
      });
      renderUpdates(search_updates, 'search');
      $('#stream').prepend('<h2 id="search_query" data-query="' + q + '">Results for "' + q + '"</h2>')
      Application.last_search = $('#stream').html();
      $('a[rel=refresh]').removeClass("disabled");
    });
    return false;
  });
  
  $('#update_form textarea').keydown(function(e) {
    if (e.keyCode == 13) {
      $('#update_form').submit();
      return false;
    } else {
      updateCounter.call(this);
    }
  }).blur(updateCounter).change(updateCounter);
  
  $('#update_form button').hover(function() {
    $(this).html('Update');
  }, function() {
    $(this).html($(this).data('count') || 'Update');
  });
  
  $('a.toggle_paste').live('click', function() {
    $(this).parents('div.update')
      .find('div.paste')
      .trigger('paste:toggle');
    return false;
  });
  
  $('div.update').live('paste:toggle', function() {
    $(this).find('div.paste').toggle();
    $(this).find('a.toggle_paste').html($(this).find('div.paste').is(':visible') ? 'Less' : 'More');
  });
  
  $('#view_options a[href="#' + localStorage['display_preference'] + '"]').click();
  });
  
  $('a[rel=reply]').live('click', function() {
    var text = '@' + $(this).attr('data-screen-name') + ' ';
    $('#update_form textarea').val(text).focus().selectRange(text.length, text.length);
  });
  
  $('a[rel=retweet]').live('click', function() {
    var update = $(this).parents('.update');
    var spinner = $('<div class="retweeting_status spinner"><img src="spinner.gif"/> Retweeting...</span>');
    update.find('.content').prepend(spinner);
    spinner.fadeIn('fast');
    Presently.retweet($(this).attr('data-id'), function() {
      spinner.html('Retweet Successful').delay(2000).fadeOut('fast', function() { 
        spinner.remove(); 
        update.find('a[rel=retweet]').remove();
        update.find('.content').append('<span class="retweeted_by you">retweeted by <strong>you</strong></span>');
      });
    });
    return false;
  });
  
  $('a[rel=refresh]').live('click', function() {
    $('div.spinner').fadeIn();
    if (current_timeline == 'search') {
      Presently.search($('h2#search_query').attr('data-query'), $('.update:first').attr('data-id'), function(search_results) {
        $('.spinner').fadeOut('fast');
        var search_updates = []
        $.each(search_results.results, function() {
          search_updates.push(Presenters.search_result(this));
        });
        var heading = $('#stream h2').remove();
        renderUpdates(search_updates, 'search');
        $('#stream').prepend(heading);
        Application.last_search = $('#stream').html();
      });
    } else {
      Application.fetchUpdates();
    }
  });